Include "TC_main.data";

If(!StrCmp(TC_PREFIX,""))
  TC_PREFIX_ = "";
Else
  TC_PREFIX_ = StrCat[TC_PREFIX, "_"];
EndIf

Function {
  I[] = Complex[0.,1.]; //i complexe
  alpha_vect[] = Vector[Cos[alpha],Sin[alpha],0]; //vecteur angle alpha
  If(!StrCmp(INCIDENT_WAVE,"Plane Wave"))
     //one plane incidente vect alpha
    uInc[] = Complex[Cos[kap*(XYZ[]*alpha_vect[])],Sin[kap*(XYZ[]*alpha_vect[])]];
    dx_uInc[] = I[]*kap*Cos[alpha]*uInc[];
  ElseIf(!StrCmp(INCIDENT_WAVE,"Eigenmode"))
    uInc[] = Cos[m_mode*Pi*(Y[]+l/2)/l];
    dx_uInc[] = I[]*kap*uInc[];
  EndIf
}

// 2 subdomains (DDM) case)
Group {
  D() = {};
  For i In {0:(NDom-1)}
    Omega~{i} = Region[{i}];
    D() += i;
    j = (i+1)%NDom; // neighbor
    D~{i} = {j};

    If (i==0)
      GammaIncoming~{i} = Region[{31}];
      GammaOutgoing~{i} = Region[{}];
    Else
      GammaOutgoing~{i} = Region[{33}];
      GammaIncoming~{i} = Region[{}];
    EndIf
    GammaTop~{i} = Region[{(100+i)}];
    GammaBottom~{i} = Region[{(200+i)}];

    //Sigma_ij = dOmega1 cap dOmega2
    Sigma~{i}~{j} = Region[{20,21,22,23}];
    Delta~{i}~{j} = Region[{20,21,22,23}];
    Sigma~{i} = Region[{20,21,22,23}];
    Delta~{i} = Region[{20,21,22,23}];

    If(CORNER_CONDITION == 0) //Dirichlet
      // Segments are considered as "only one"
      r = 0; // only one "segment"
      NDelta~{i}~{j} = 1;
      Delta~{i}~{j}~{r} = Region[{20,21,22,23}];
      dDelta~{i}~{j}~{r} = Region[{}];
      ListdDelta~{i}~{j}~{r}() = {}; // No corner point (empty list)
      ListdDelta~{i}~{j}() = {}; // No corner point (empty list)
    Else
      // Sigma_{ij} is split into segments ("broken" H^1(Sigma_ij))
      NDelta~{i}~{j} = 4;
      // if i == 0 then r = {3,2,1,0} 
      // else i == 1 then r = {0,1,2,3} 
      For rr In {0:(NDelta~{i}~{j}-1)}
        r = (i==0)?(NDelta~{i}~{j}-1-rr):rr;
        s = (i==0)?(rr):(NDelta~{i}~{j}-1-rr); // indice of this segment from the other side

        ListdDelta~{i}~{j}~{r}() = {}; // contains the indices r' of the adjacent segments (possibly unique)
        Delta~{i}~{j}~{r} = Region[{(20+rr)}];

        //Nods Q^ij_r = Delta^ij_r \cap Delta^ij_r'
        If (r==0) // First segment
          next = r+1;
          Q~{i}~{j}~{r}~{next} = Region[{((i==0)?53-r:51+r)}];
          dDelta~{i}~{j}~{r}~{0} = Region[{((i==0)?53-r:51+r)}];
          ListdDelta~{i}~{j}~{r}() += next; // next segment

          dDelta~{i}~{j}~{r} = Region[{((i==0)?53-r:51+r)}];
        ElseIf (r==1) // second segment
          next = r+1;
          Q~{i}~{j}~{r}~{next} = Region[{((i==0)?53-r:51+r)}];
          dDelta~{i}~{j}~{r}~{0} = Region[{((i==0)?53-r:51+r)}];
          ListdDelta~{i}~{j}~{r}() += next; // next segment

          prev = r-1;
          Q~{i}~{j}~{r}~{prev} = Region[{((i==0)?53-prev:51+prev)}];
          dDelta~{i}~{j}~{r}~{1} = Region[{((i==0)?53-prev:51+prev)}];
          ListdDelta~{i}~{j}~{r}() += prev; // next segment

          dDelta~{i}~{j}~{r} = Region[{((i==0)?53-r:51+r), ((i==0)?53-prev:51+prev)}];
        ElseIf (r==2) // third segment
          next = r+1;
          Q~{i}~{j}~{r}~{next} = Region[{((i==0)?53-r:51+r)}];
          dDelta~{i}~{j}~{r}~{0} = Region[{((i==0)?53-r:51+r)}];
          ListdDelta~{i}~{j}~{r}() += next; // next segment

          prev = r-1;
          Q~{i}~{j}~{r}~{prev} = Region[{((i==0)?53-prev:51+prev)}];
          dDelta~{i}~{j}~{r}~{1} = Region[{((i==0)?53-prev:51+prev)}];
          ListdDelta~{i}~{j}~{r}() += prev; // next segment

          dDelta~{i}~{j}~{r} = Region[{((i==0)?53-r:51+r), ((i==0)?53-prev:51+prev)}];
        Else // last segment
          prev = r-1;
          Q~{i}~{j}~{r}~{prev} = Region[{((i==0)?53-prev:51+prev)}];
          dDelta~{i}~{j}~{r}~{0} = Region[{((i==0)?53-prev:51+prev)}];
          ListdDelta~{i}~{j}~{r}() += prev; // next segment

          dDelta~{i}~{j}~{r} = Region[{((i==0)?53-prev:51+prev)}];
        EndIf
      EndFor
    EndIf
  EndFor
}

Group{
  Omega = Region[{(0:(NDom-1))}];

  Sigma = Region[{20,21,22,23}];
  Delta = Region[{20,21,22,23}];
  
  GammaIncoming = Region[{31}];
  GammaOutgoing = Region[{33}];
  GammaTop = Region[{100, 101}];
  GammaBottom = Region[{200, 201}];
}


Include "TC_Decomposition.pro";
Include "TC_WaveGuide.pro";
Include "TC_WaveGuide_DDM.pro";
